/*
 * Implement Tony Murray's tclLDAP extension as a Tcl package
 *
 * v1.0    8-May-98 Gareth Owen (oweng@aston.ac.uk)
 *  1.1   15-Dec-98
 *  1.1.1 20-May-99 Bug fix (LdapAdd not registered) 
 *  1.2    2-Apr-01 
 */

#include <lber.h>
#include <ldap.h>
#include <tcl.h>
#include "tclLdap.h"


#define LdapPkgVersion "1.1.1"
int ldcnt = -1;
struct LDAP *ldprime[1024];

/* Keeps track of used handles so we can reuse them */
char usedcon[1024];

int Tcl_LdapSearchCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapBindCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapUnBindCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapDeleteCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapModifyCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapModRDNCmd(ClientData dummy,Tcl_Interp *interp,
		      int argc,char **argv);
int Tcl_LdapInitCmd(ClientData dummy,Tcl_Interp *interp,
		    int argc,char **argv);


static void InitVars(Tcl_Interp *interp)
{

  char str[256];

  
  Tcl_SetVar2(interp, "ldap","searchbase",LDAPSEARCH_BASE,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","binddn,search",\
	      LDAPSEARCH_BINDDN,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","binddn,modify",\
	      LDAPMODIFY_BINDDN,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","binddn,delete",\
	      LDAPDELETE_BINDDN,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","binddn,modrdn",\
	      LDAPMODRDN_BINDDN,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","host",LDAPHOST,TCL_GLOBAL_ONLY);
  sprintf(str,"%d",LDAP_PORT);
  Tcl_SetVar2(interp, "ldap","port",str,TCL_GLOBAL_ONLY);
  sprintf(str,"%d",LDAP_VERSION);
  Tcl_SetVar2(interp, "ldap","version",str,TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "ldap","dn","0",TCL_GLOBAL_ONLY);

}

static int InitCmds(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp,"LdapInit",Tcl_LdapInitCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapSearch",Tcl_LdapSearchCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapBind",Tcl_LdapBindCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapUnBind",Tcl_LdapUnBindCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapDelete",Tcl_LdapDeleteCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapModRDN",Tcl_LdapModRDNCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapModify",Tcl_LdapModifyCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp,"LdapAdd",Tcl_LdapModifyCmd,
		    (ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

  return TCL_OK;
}


int Ldap_Init(Tcl_Interp *interp)
{
  int code,i;

  for (i=0;i<1024;i++) usedcon[i]=0;
  
  code = Tcl_PkgProvide(interp,"Ldap",LdapPkgVersion);
  
  if (code == TCL_OK) {
    InitVars(interp);
    InitCmds(interp);
  }

  return code;

}

/*
 * Dummy function to initialise. Package commands and variables don't
 * come into existence till a command is invoked.
 */
Tcl_LdapInitCmd(ClientData dummy,Tcl_Interp *interp,int argc,char **argv)
{
  return TCL_OK;
}
